/* gcc -g -Wall -o xcb_eye xcb_eye.c `pkg-config --cflags --libs xcb xcb-aux xcb-image` */

#include <stdlib.h>
#include <stdio.h>

#include <xcb/xcb.h>
#include <xcb/shm.h>
#include <xcb/xcb_aux.h>
#include <xcb/xcb_image.h>

#include "eyes.bit"
#include "eyesmask.bit"

void
quit (xcb_connection_t *c, xcb_generic_event_t *e)
{
  if (e)
    free (e);
  xcb_disconnect (c);

  exit (EXIT_SUCCESS);
}

xcb_pixmap_t
xcb_create_bitmap_from_data (xcb_connection_t    *c,
                             xcb_drawable_t       draw,
                             const unsigned char *data,
                             unsigned int         width,
                             unsigned int         height)
{
  xcb_pixmap_t   pixmap;
  xcb_image_t    image;
  xcb_gcontext_t gc;

  pixmap = xcb_generate_id (c);
  xcb_create_pixmap (c,
                     1,
                     pixmap,
                     draw,
                     width,
                     height);

  gc = xcb_generate_id (c);
  xcb_create_gc (c, gc, pixmap, 0, NULL);
  
  image.width = width;
  image.height = height;
  image.depth = 1;
  image.bits_per_pixel = 1;
  image.xoffset = 0;
  image.format = XCB_IMAGE_FORMAT_XY_PIXMAP;
  image.data = (unsigned char *)data;
  image.image_byte_order = XCB_IMAGE_ORDER_LSB_FIRST;
  image.bitmap_format_scanline_unit = 8;
  image.bitmap_format_bit_order = XCB_IMAGE_ORDER_LSB_FIRST;
  image.bitmap_format_scanline_pad = 8;
  image.bytes_per_line = (eyes_width + 7) >> 3;
  
  xcb_image_put (c,
                 pixmap,
                 gc,
                 &image,
                 0, 0,
                 0, 0,
                 width, height);
  xcb_free_gc (c, gc);

  return pixmap;
}

int
main (int argc, char *argv[])
{
  xcb_screen_iterator_t screen_iter;
  xcb_connection_t     *c;
  const xcb_setup_t    *setup;
  xcb_screen_t         *screen;
  xcb_generic_event_t  *e;
  xcb_generic_error_t  *error;
  xcb_void_cookie_t     cookie_window;
  xcb_void_cookie_t     cookie_map;
  xcb_window_t          window;
  xcb_gcontext_t        gc;
  uint32_t              mask;
  uint32_t              values[2];
  int                   screen_number;
  xcb_pixmap_t          eyes_pixmap;
  xcb_pixmap_t          eyes_mask;

  /* Connection */
  c = xcb_connect (NULL, &screen_number);
  if (!c) {
    fprintf (stderr, "ERROR: can't connect to an X server\n");
    return -1;
  }

  /* getting the current screen */
  setup = xcb_get_setup (c);

  screen = NULL;
  screen_iter = xcb_setup_roots_iterator (setup);
  for (; screen_iter.rem != 0; --screen_number, xcb_screen_next (&screen_iter))
    if (screen_number == 0)
      {
        screen = screen_iter.data;
        break;
      }
  if (!screen) {
    fprintf (stderr, "ERROR: can't get the current screen\n");
    xcb_disconnect (c);
    return -1;
  }

  /* Main window */
  window = xcb_generate_id (c);
  mask = XCB_CW_BACK_PIXEL | XCB_CW_EVENT_MASK;
  values[0] = screen->white_pixel;
  values[1] =
    XCB_EVENT_MASK_KEY_RELEASE |
    XCB_EVENT_MASK_EXPOSURE;
  cookie_window = xcb_create_window_checked (c,
                                             screen->root_depth,
                                             window, screen->root,
                                             20, 200,
                                             eyes_width, eyes_height,
                                             0, XCB_WINDOW_CLASS_INPUT_OUTPUT,
                                             screen->root_visual,
                                             mask, values);
  cookie_map = xcb_map_window_checked (c, window);

  /* error managing */
  error = xcb_request_check (c, cookie_window);
  if (error) {
    fprintf (stderr, "ERROR: can't create window : %d\n", error->error_code);
    xcb_disconnect (c);
    return -1;
  }
  error = xcb_request_check (c, cookie_map);
  if (error) {
    fprintf (stderr, "ERROR: can't map window : %d\n", error->error_code);
    xcb_disconnect (c);
    return -1;
  }

  xcb_flush(c);

  eyes_pixmap = xcb_generate_id (c);
  xcb_create_pixmap (c,
                     screen->root_depth,
                     eyes_pixmap,
                     window,
                     eyes_width,
                     eyes_height);

  eyes_mask = xcb_generate_id (c);
  xcb_create_pixmap (c,
                     screen->root_depth,
                     eyes_mask,
                     window,
                     eyesmask_width,
                     eyesmask_height);

  gc = xcb_generate_id (c);
  xcb_create_gc (c, gc, window, 0, NULL);

  while (1) {
    e = xcb_poll_for_event(c);
    if (e) {
      switch (e->response_type) {
      case XCB_EXPOSE: {
        xcb_pixmap_t pixmap;
        
        printf ("Expose event\n");
        pixmap = xcb_create_bitmap_from_data (c,
                                              screen->root,
                                              eyes_bits,
                                              eyes_width,
                                              eyes_height);

        xcb_copy_area (c,
                       pixmap,
                       window,
                       gc,
                       0, 0,
                       0, 0,
                       eyes_width, eyes_height);

        xcb_flush (c);
        break;
      }
      case XCB_KEY_RELEASE: {
        xcb_key_release_event_t *ev;

        ev = (xcb_key_release_event_t *)e;

        switch (ev->detail) {
          /* ESC */
        case 9:
          free (e);
          xcb_disconnect (c);
          return 0;
        }
      }
      }
      free (e);
    }
  }

  return 1;
}
